--author: igor29381
--v1.0
--date 13.01.18

GuardPost = {};
GuardPost_mt = Class(GuardPost, Placeable);
InitObjectClass(GuardPost, "GuardPost");
GuardPost.curModDir = g_currentModDirectory;

function GuardPost:new(isServer, isClient, customMt)
	local mt = customMt;
	if mt == nil then
		mt = GuardPost_mt;
	end;
	local self = Placeable:new(isServer, isClient, mt);
	registerObjectClassName(self, "GuardPost");
	self.guardPost = true;
	return self;
end;

function GuardPost:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
	if not GuardPost:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
		return false;
	end;
	if x ~= 0 and z ~= 0 then
		local splineNode = getChild(self.nodeId, "spline");
		if splineNode > 0 then
			self.splineNode = splineNode;
			self.splineLength = getSplineLength(splineNode);
			self.splinePosition = 0;
		end;
		local xmlFile = loadXMLFile("TempXML", xmlFilename);
		local i3dFilename = GuardPost.curModDir..getXMLString(xmlFile, "placeable.i3dFilename");
		if fileExists(i3dFilename) then
			local guardRoot = Utils.loadSharedI3DFile(i3dFilename);
			self.skeletonNode = getChildAt(guardRoot, 0);
			self.firstSkinNode = getChildAt(guardRoot, 1);
			self.secondSkinNode = getChildAt(guardRoot, 2);
			self.guardGroup = createTransformGroup("guardGroup");
			link(self.guardGroup, self.skeletonNode);
			link(getRootNode(), self.firstSkinNode);
			link(getRootNode(), self.secondSkinNode);
			delete(guardRoot);
			setVisibility(self.firstSkinNode, false);
			setVisibility(self.secondSkinNode, false);
			self.clipDistance = Utils.getNoNil(getXMLInt(xmlFile, "placeable.character#playClipDistance"), 200);
			self.visibility = false;
			self.animCharSet = getAnimCharacterSet(self.skeletonNode);
			if self.animCharSet ~= 0 then
				self.isEnabled = true;
				local clips = getXMLString(xmlFile, "placeable.character#clips");
				clips = Utils.splitString(" ", clips);
				local walkClips = getXMLString(xmlFile, "placeable.character#walkClips");
				self.walkTracks = {Utils.getVectorFromString(walkClips)};
				self.numTracks = 0;
				for i=1, #clips do
					local clip = getAnimClipIndex(self.animCharSet, clips[i]);
					if clip > -1 then
						assignAnimTrackClip(self.animCharSet, i, clip);
						setAnimTrackLoopState(self.animCharSet, i, true);
						setAnimTrackSpeedScale(self.animCharSet, i, 1);
						self.numTracks = self.numTracks + 1;
					else
						print("GuardPost: clip "..clips[i].." not found");
					end;
				end;
				self.activeTrack = 1;
				self.nextTrack = 1;
				self.timer1 = 0;
				self.timer2 = 0;
			end;
			local soundFile = getXMLString(xmlFile, "placeable.soundFile");
			if soundFile then
				self.sound = createAudioSource("sound", GuardPost.curModDir..soundFile, 30, 15, 1, 10000);
				local sample = getAudioSourceSample(self.sound);
				self.soundDuration = getSampleDuration(sample);
				link(getRootNode(), self.sound);
				setVisibility(self.sound, false);
				self.soundTimer = math.random(15000, 60000);
			end;
		end;
		delete(xmlFile);
		self.position = {x=x, z=z};
		if g_server and Russia then
			local base = Russia.Economica.RandomEventsBase;
			if base and x~=0 and z~=0 then
				table.insert(base.protectedAreas, {x1=x-150, z1=z-150, x2=x+150, z2=z+150});
				self.areaIndex = #base.protectedAreas;
			end;
			if self:getRussiaBaseIsNear() then
				Russia.Economica.randomEventsByNames.moneyTheft.active = true;
			end;
		end;
		local borderNode = getChild(self.nodeId, "border");
		if borderNode > 0 then
			self.borderNode = borderNode;
			link(getRootNode(), borderNode);
			setTranslation(borderNode, x, y, z);
			setVisibility(borderNode, false);
		end;
		local visitPoint = getChild(self.nodeId, "visitPoint");
		if visitPoint > 0 then
			x, _, z = getWorldTranslation(visitPoint);
		end;
		local hotspotFileName = GuardPost.curModDir.."textures/hotspot.dds";
		local width, height = getNormalizedScreenValues(10, 10);
		self.mapHotspot = g_currentMission.ingameMap:createMapHotspot("", "", hotspotFileName, nil, nil, x, z, width, height, false, false, true, nil, true);
		self.text = {};
		self.text[true] = g_i18n:getText("hideBorders");
		self.text[false] = g_i18n:getText("renderBorders");
		self.isEnabled = true;
	end;
	return true;
end;

function GuardPost:delete()
	if self.areaIndex and Russia then
		local base = Russia.Economica.RandomEventsBase;
		table.remove(base.protectedAreas, self.areaIndex);
	end;
	if self.borderNode then
		delete(self.borderNode);
	end;
	if self.skeletonNode then
		delete(self.skeletonNode);
		delete(self.firstSkinNode);
		delete(self.secondSkinNode);
	end;
	unregisterObjectClassName(self);
	GuardPost:superClass().delete(self);
end;

function GuardPost:deleteFinal()
	GuardPost:superClass().deleteFinal(self);
end;

function GuardPost:readStream(streamId, connection)
	GuardPost:superClass().readStream(self, streamId, connection);
end;

function GuardPost:writeStream(streamId, connection)
	GuardPost:superClass().writeStream(self, streamId, connection);
end;

function GuardPost:update(dt)
	if self.isEnabled then
		local x,y,z = getSplinePosition(self.splineNode, self.splinePosition);
		if self:getCameraIsNear(x,y,z) then
			if not self.visibility then
				self:setVisibility(true);
				enableAnimTrack(self.animCharSet, self.activeTrack);
				setAnimTrackBlendWeight(self.animCharSet, self.activeTrack, 1);
			end;
			self.timer1 = self.timer1 - dt;
			if self.timer1 <= 0 then
				self.timer1 = math.random(5000, 15000);
				self.nextTrack = math.random(1, self.numTracks);
				enableAnimTrack(self.animCharSet, self.nextTrack);
			end;
			if self.nextTrack ~= self.activeTrack then
				self.timer2 = math.min(self.timer2 + 0.05, 1);
				setAnimTrackBlendWeight(self.animCharSet, self.nextTrack, self.timer2);
				if isAnimTrackEnabled(self.animCharSet, self.activeTrack) then
					local ATBW = 1 - getAnimTrackBlendWeight(self.animCharSet, self.nextTrack);
					setAnimTrackBlendWeight(self.animCharSet, self.activeTrack, ATBW);
				end;
				if self.timer2 == 1 then
					self.timer2 = 0;
					disableAnimTrack(self.animCharSet, self.activeTrack);
					setAnimTrackBlendWeight(self.animCharSet, self.activeTrack, 1);
					self.activeTrack = self.nextTrack;
				end;
			end;
			if self.walkTracks[self.activeTrack] > 0 or self.walkTracks[self.nextTrack] > 0 then
				local speed = self.walkTracks[self.activeTrack];
				if self.walkTracks[self.nextTrack] > self.walkTracks[self.activeTrack] then
					speed = Utils.clamp(self.walkTracks[self.nextTrack]*self.timer2, self.walkTracks[self.activeTrack], self.walkTracks[self.nextTrack]);
				elseif self.walkTracks[self.nextTrack] < self.walkTracks[self.activeTrack] then
					speed = math.max(self.walkTracks[self.activeTrack]*(1 - self.timer2), self.walkTracks[self.nextTrack]);
				end;
				local timeScale = (speed/3.6) / self.splineLength;
				if self.parent then
					self.splinePosition = self.parent.splinePosition;
				else
					self.splinePosition = self.splinePosition - 0.001*dt*timeScale;
				end;
			end;
			local rx,ry,rz = getSplineOrientation(self.splineNode, self.splinePosition, 0, -1, 0);
			setTranslation(self.guardGroup, x, y, z);
			setRotation(self.guardGroup, rx,ry,rz);
			if self.sound then
				self.soundTimer = self.soundTimer - dt;
				if self.soundTimer <= 0 then
					if getVisibility(self.sound) then
						setVisibility(self.sound, false);
						self.soundTimer = math.random(15000, 60000);
					else
						setTranslation(self.sound, x, y, z);
						setVisibility(self.sound, true);
						self.soundTimer = self.soundDuration+500;
					end;
				end;
			end;
		else
			if self.visibility then
				self:setVisibility(false);
				disableAnimTrack(self.animCharSet, self.activeTrack);
				disableAnimTrack(self.animCharSet, self.nextTrack);
			end;
		end;
		if self.borderNode and self:getPlayerIsNear() then
			local currentVisibility = getVisibility(self.borderNode);
			g_currentMission:addHelpButtonText(self.text[currentVisibility], InputBinding.IMPLEMENT_EXTRA4);
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
				setVisibility(self.borderNode, not currentVisibility);
			end;
		end;
	end;
end;

function GuardPost:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not GuardPost:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false;
	end;
	return true;
end;

function GuardPost:getSaveAttributesAndNodes(nodeIdent)
	local attributes, nodes = GuardPost:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	return attributes, nodes;
end;

function GuardPost:getCameraIsNear(x,y,z)
	local cx,cy,cz = getWorldTranslation(getCamera());
	return Utils.vector3Length(x-cx, y-cy, z-cz) < self.clipDistance;
end;

function GuardPost:setVisibility(state)
	setVisibility(self.firstSkinNode, state);
	setVisibility(self.secondSkinNode, state);
	self.visibility = state;
end;

function GuardPost:getPlayerIsNear()
	if g_currentMission.controlPlayer then
		local x,_,z = getWorldTranslation(g_currentMission.player.rootNode);
		return Utils.vector2Length(x-self.position.x, z-self.position.z) < 4;
	end;
	return false;
end;

function GuardPost:getRussiaBaseIsNear()
	local x,_,z = getWorldTranslation(Russia.UniversalFactory.Farming.nodeId);
	return Utils.vector2Length(x-self.position.x, z-self.position.z) < 300;
end;

registerPlaceableType("GuardPost", GuardPost);

function drawGuardPost()
	if Russia and not g_placementScreen.isSellMode then
		local placeable = g_placementScreen.placeable;
		if placeable and placeable.guardPost then
			local x1,_,z1 = getWorldTranslation(Russia.UniversalFactory.Farming.nodeId);
			local x,_,z = getWorldTranslation(placeable.nodeId);
			local distance = Utils.vector2Length(x-x1, z-z1);
			setTextAlignment(RenderText.ALIGN_CENTER);
			setTextColor(1, 1, 1, 1);
			if distance < 300 then
				setTextColor(0, 1, 0, 1);
			end;
			renderText(0.5, 0.25, 0.015, string.format(g_i18n:getText("guardPostDistance"), distance));
			setTextAlignment(RenderText.ALIGN_CENTER);
			setTextColor(1, 1, 1, 1);
		end;
	end;
end;

g_placementScreen.draw = Utils.appendedFunction(g_placementScreen.draw, drawGuardPost);
